/*
   Make the definitions of the Comparator classes
   into nested anonymous inner classes.
*/
import java.util.*;

public class ComparatorTester_3
{
   public static void main(String[] args)
   {
      List<Country> countries = Arrays.asList(
         new Country("Uruguay",   176220,   3.4),
         new Country("Thailand",  514000,  67.0),
         new Country("Belgium",    30528,  11.0),
         new Country("Zaire",    2345410,  46.5),
         new Country("Belize",     22966,   0.334));


      // These comparators are now anonymous inner classes.
      // NOTE: The inner classes are anonymous, but the
      // comparator objects are named comp1 through comp6
      // (so we have named instances of anonymous classes).
      Comparator<Country> comp1 = new
         Comparator<Country>()
         {
            public int compare(Country country1, Country country2)
            {
               return country1.getName().compareTo(country2.getName());
            }
         };

      Comparator<Country> comp2 = new
         Comparator<Country>()
         {
            public int compare(Country country1, Country country2)
            {
               return country2.getName().compareTo(country1.getName());
            }
         };

      Comparator<Country> comp3 = new
         Comparator<Country>()
         {
            public int compare(Country country1, Country country2)
            {
               if (country1.getArea() < country2.getArea()) return -1;
               if (country1.getArea() > country2.getArea()) return  1;
               return 0;
            }
         };

      Comparator<Country> comp4 = new
         Comparator<Country>()
         {
            public int compare(Country country1, Country country2)
            {
               if (country1.getArea() > country2.getArea()) return -1;
               if (country1.getArea() < country2.getArea()) return  1;
               return 0;
            }
         };

      Comparator<Country> comp5 = new
         Comparator<Country>()
         {
            public int compare(Country country1, Country country2)
            {
               if (country1.getPopulation() < country2.getPopulation()) return -1;
               if (country1.getPopulation() > country2.getPopulation()) return  1;
               return 0;
            }
         };

      Comparator<Country> comp6 = new
         Comparator<Country>()
         {
            public int compare(Country country1, Country country2)
            {
               if (country1.getPopulation() > country2.getPopulation()) return -1;
               if (country1.getPopulation() < country2.getPopulation()) return  1;
               return 0;
            }
         };

      // Call the sort() method and pass it a comparator object.

      Collections.sort(countries, comp1);
      // Now the List is sorted by country name, ascending.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();

      Collections.sort(countries, comp2);
      // Now the List is sorted by country name, descending.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();

      Collections.sort(countries, comp3);
      // Now the List is sorted by area, ascending.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();

      Collections.sort(countries, comp4);
      // Now the List is sorted by area, descending.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();

      Collections.sort(countries, comp5);
      // Now the List is sorted by population, ascending.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();

      Collections.sort(countries, comp6);
      // Now the List is sorted by population, descending.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();
   }
}


/**
   A country with a name, area, and population.
*/
class Country
{
   private String name;
   private double area;
   private double population;

   /**
      Constructs a country.
      @param name the name of the country
      @param area the area of the country
      @param population  the population of the country
   */
   public Country(String name, double area, double population)
   {
      this.name = name;
      this.area = area;
      this.population = population;
   }

   /**
      Gets the name of the country.
      @return the name
   */
   public String getName()
   {
      return this.name;
   }

   /**
      Gets the area of the country.
      @return the area
   */
   public double getArea()
   {
      return this.area;
   }

   /**
      Gets the population of the country.
      @return the area
   */
   public double getPopulation()
   {
      return this.population;
   }

   public String toString()
   {
      return name + ": area = " + area + ", population = " + population;
   }
}