/*
   Make the definitions of the Comparator classes
   into anonymous inner class at the call to sort().
*/
import java.util.*;

public class ComparatorTester_4
{
   public static void main(String[] args)
   {
      List<Country> countries = Arrays.asList(
         new Country("Uruguay",   176220,   3.4),
         new Country("Thailand",  514000,  67.0),
         new Country("Belgium",    30528,  11.0),
         new Country("Zaire",    2345410,  46.5),
         new Country("Belize",     22966,   0.334));


      // Call sort() and pass it a Comparator that is an
      // anonymous instance of an anonymous inner class.
      Collections.sort(countries, new
         Comparator<Country>()
         {
            public int compare(Country country1, Country country2)
            {
               return country1.getName().compareTo(country2.getName());
            }
         });
      // Now the List is sorted by name, ascending.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();


      Collections.sort(countries, new
         Comparator<Country>()
         {
            public int compare(Country country1, Country country2)
            {
               return country2.getName().compareTo(country1.getName());
            }
         });
      // Now the List is sorted by name, descending.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();


      Collections.sort(countries, new
         Comparator<Country>()
         {
            public int compare(Country country1, Country country2)
            {
               if (country1.getArea() < country2.getArea()) return -1;
               if (country1.getArea() > country2.getArea()) return  1;
               return 0;
            }
         });
      // Now the List is sorted by area, ascending.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();


      Collections.sort(countries, new
         Comparator<Country>()
         {
            public int compare(Country country1, Country country2)
            {
               if (country1.getArea() > country2.getArea()) return -1;
               if (country1.getArea() < country2.getArea()) return  1;
               return 0;
            }
         });
      // Now the List is sorted by area, descending.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();


      Collections.sort(countries, new
         Comparator<Country>()
         {
            public int compare(Country country1, Country country2)
            {
               if (country1.getPopulation() < country2.getPopulation()) return -1;
               if (country1.getPopulation() > country2.getPopulation()) return  1;
               return 0;
            }
         });
      // Now the List is sorted by population, ascending.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();


      Collections.sort(countries, new
         Comparator<Country>()
         {
            public int compare(Country country1, Country country2)
            {
               if (country1.getPopulation() > country2.getPopulation()) return -1;
               if (country1.getPopulation() < country2.getPopulation()) return  1;
               return 0;
            }
         });
      // Now the List is sorted by population, descending.
      for (Country c : countries)
        System.out.println( c );
      System.out.println();
   }
}



/**
   A country with a name, area, and population.
*/
class Country
{
   private String name;
   private double area;
   private double population;

   /**
      Constructs a country.
      @param name the name of the country
      @param area the area of the country
      @param population  the population of the country
   */
   public Country(String name, double area, double population)
   {
      this.name = name;
      this.area = area;
      this.population = population;
   }

   /**
      Gets the name of the country.
      @return the name
   */
   public String getName()
   {
      return this.name;
   }

   /**
      Gets the area of the country.
      @return the area
   */
   public double getArea()
   {
      return this.area;
   }

   /**
      Gets the population of the country.
      @return the area
   */
   public double getPopulation()
   {
      return this.population;
   }

   public String toString()
   {
      return name + ": area = " + area + ", population = " + population;
   }
}