(define countries
   '(("Uruguay"  176220   3.4)
     ("Thailand" 513000    67)
     ("Belgium"   30528    11)
     ("Zaire"   2345410  46.5) 
     ("Belize"    22966  0.334)))

(define sort
  (lambda (lst cmp)  ; cmp is a function parameter
    (letrec
        ((insert
          (lambda (x lst)  ; nested function
            (cond
              ((null? lst) (cons x '()))
              ((cmp x (car lst)) (cons x lst)) ; cmp is a non-local reference
              (else                            ; that is lexically scoped
               (cons (car lst) (insert x (cdr lst)))))))
         (nestedSort
          (lambda (lst)                ; another nested function
            (cond ((null? lst) '())
            (else (insert (car lst) (nestedSort (cdr lst))))))))
      (nestedSort lst))))

(define makeCmpByArea
  (lambda (fn)                              ; fn is a function parameter
    (lambda (c1 c2)                         ; makeCmpByArea has a function return value
      (fn (car (cdr c1)) (car (cdr c2)))))) ; fn is a non-local reference

(define makeCmpByPopulation
  (lambda (fn)                       ; fn is a function parameter
    (lambda (c1 c2)                  ; a function return value
       (fn (caddr c1) (caddr c2))))) ; fn is a non-local reference

(sort countries (makeCmpByArea <))       ; (sort countries (lambda (c1 c2) (< (cadr c1)  (cadr c2))))
(sort countries (makeCmpByArea >))       ; (sort countries (lambda (c1 c2) (> (cadr c1)  (cadr c2))))
(sort countries (makeCmpByPopulation <)) ; (sort countries (lambda (c1 c2) (< (caddr c1) (caddr c2))))
(sort countries (makeCmpByPopulation >)) ; (sort countries (lambda (c1 c2) (> (caddr c1) (caddr c2))))

; Make our Scheme example look more like Java 8.
(define getA (lambda (country) (cadr  country)))
(define getP (lambda (country) (caddr country)))
(sort countries (lambda (c1 c2) (< (getA c1) (getA c2))))
(sort countries (lambda (c1 c2) (> (getA c1) (getA c2))))
(sort countries (lambda (c1 c2) (< (getP c1) (getP c2))))
(sort countries (lambda (c1 c2) (> (getP c1) (getP c2))))