(define countries
   '(("Uruguay"  176220   3.4)
     ("Thailand" 513000    67)
     ("Belgium"   30528    11)
     ("Zaire"   2345410  46.5) 
     ("Belize"    22966  0.334)))

(define sort
  (lambda (lst cmp)  ; cmp is a function parameter
    (letrec
        ((insert
          (lambda (x lst)  ; nested function
            (cond
              ((null? lst) (cons x '()))
              ((cmp x (car lst)) (cons x lst)) ; cmp is a non-local reference
              (else                            ; that is lexically scoped
               (cons (car lst) (insert x (cdr lst)))))))
         (nestedSort
          (lambda (lst)                ; another nested function
            (cond ((null? lst) '())
            (else (insert (car lst) (nestedSort (cdr lst))))))))
      (nestedSort lst))))

; The following function creates comparators that are parameterized
; two ways. The fn1 input determines if the returned comparator is
; an ascending or descending comparator (fn1 will usually have the
; value < or >). The fn2 input determines if the returned comparator
; looks at the area or population fields in a country list (fn2 will 
; usualy have the value getA or getP).
(define makeCmp                    ; make a doubly parameterized comparator
  (lambda (fn1 fn2)                ; fn1 and fn2 are function parameters
    (lambda (c1 c2)                ; a nested, function return value
       (fn1 (fn2 c1) (fn2 c2)))))  ; fn1 and fn2 are non-local references

(define getA (lambda (country) (cadr  country)))  ; get the area value out of a country list
(define getP (lambda (country) (caddr country)))  ; get the population value out of a country list

(sort countries (makeCmp < getA))  ; sort ascending by area
(sort countries (makeCmp > getA))  ; sort descending by area
(sort countries (makeCmp < getP))  ; sort ascending by population
(sort countries (makeCmp > getP))  ; sort descending by population


; We can also define getA and getP this way.
(define getA cadr)   ; get the area value out of a country list
(define getP caddr)  ; get the population value out of a country list

(sort countries (makeCmp < getA))  ; sort ascending by area
(sort countries (makeCmp > getA))  ; sort descending by area
(sort countries (makeCmp < getP))  ; sort ascending by population
(sort countries (makeCmp > getP))  ; sort descending by population